/* Soma - Copyright (C) 2003-5 bakunin - Andrea Marchesini 
 *                                       <bakunin@autistici.org>
 *
 * This source code is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Public License as published 
 * by the Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version.
 *
 * This source code is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * Please refer to the GNU Public License for more details.
 *
 * You should have received a copy of the GNU Public License along with
 * this source code; if not, write to:
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * This program is released under the GPL with the additional exemption that
 * compiling, linking, and/or using OpenSSL is allowed.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#else
# error Use configure; make; make install
#endif

#include "somad.h"

void
nextsong_path_add (char *data)
{
  static char buf[SOMA_MAX_BUFF];

  SOMA_LOG_ALL ("NextSong Path Adding: %s", data);

  if (!data)
    return;

  snprintf (buf, sizeof (buf), "%c%s", DISTRIBUITED_CHAR, data);

  nextsong_add (buf);
}

void
nextsong_set_mix (int a)
{
  SOMA_LOG_ALL ("NextSong Set Mix to %d", a);

  conf->nextsong_spot = a;
}

void
nextsong_add (char *data)
{

  struct nextsong *ns;
  struct nextsong *tmp;
  struct stat st;

  SOMA_LOG_ALL ("NextSong Add %s", data);

  if (!data)
    return;

  if (!remote_file (data) && !distribuited_file (data)
      && (lstat (data, &st) || !S_ISREG (st.st_mode)))
    {
      SOMA_LOG_WARN ("%s file does not exist!", data);
      return;
    }

  if (!(ns = (struct nextsong *) malloc (sizeof (struct nextsong))))
    SOMA_FATAL ("Error: memory.");

  ns->next = NULL;
  if (!(ns->song = strdup (data)))
    SOMA_FATAL ("Error: memory.");

  LOCK ();

  if (!conf->next_song)
    {
      conf->next_song = ns;
      UNLOCK ();
      return;
    }

  tmp = conf->next_song;
  while (tmp->next)
    tmp = tmp->next;

  tmp->next = ns;

  UNLOCK ();
}

void
nextsong_remove (void)
{
  struct nextsong *ns;

  SOMA_LOG_ALL ("NextSong Remove");

  if (!conf->next_song)
    return;

  LOCK ();

  ns = conf->next_song;
  conf->next_song = conf->next_song->next;

  free (ns->song);
  free (ns);

  UNLOCK ();
}

void
nextsong_remove_n (int n)
{
  struct nextsong *ns, *old = NULL;

  SOMA_LOG_ALL ("NextSong Remove Number %d", n);

  if (!conf->next_song)
    return;

  LOCK ();

  ns = conf->next_song;
  while (ns)
    {
      if (!n)
	{
	  if (!old)
	    conf->next_song = conf->next_song->next;
	  else
	    old->next = ns->next;

	  free (ns->song);
	  free (ns);
	  break;
	}

      n--;

      old = ns;
      ns = ns->next;
    }

  UNLOCK ();
}

void
nextsong_remove_first (void)
{
  SOMA_LOG_ALL ("NextSong Remove First");

  nextsong_remove_n (0);
}

void
nextsong_remove_last (void)
{
  struct nextsong *ns, *old = NULL;

  SOMA_LOG_ALL ("NextSong Remove Last");

  if (!conf->next_song)
    return;

  LOCK ();

  ns = conf->next_song;
  while (ns)
    {

      if (!ns->next)
	{
	  if (!old)
	    conf->next_song = conf->next_song->next;
	  else
	    old->next = ns->next;

	  free (ns->song);
	  free (ns);
	  break;

	}

      old = ns;
      ns = ns->next;
    }

  UNLOCK ();
}

/* EOF */
